from cml.domain.reconstruction import (
    split,
    valid_reconstructed,
    calc_reliability,
    Metadata,
    PragmaticMachineLearningModel)


MODEL_COUNTERS = None


def set_valid_counter(length):
    global MODEL_COUNTERS
    MODEL_COUNTERS = [i for i in range(1, length+1)]


def reconstruct_in_parallel(*args):
    dto = args[0]
    train_block, eval_block = split(dto.learnblock, dto.settings.reliability_sample)
    trained_model = dto.model.train(train_block.as_numpy_array(),
                                    [i for i in train_block.get_column("Z")])
    if valid_reconstructed(trained_model, "conceptual", dto.settings):
        reliability = calc_reliability(trained_model, eval_block, "nominal")
        if reliability >= dto.settings.min_reliability:
            trained_model.reliability = reliability
            zeta = ".".join([dto.abv,
                            str(dto.tier),
                            dto.learnblock.subject])
            meta = Metadata(dto.abv,
                            dto.tier,
                            dto.count,
                            dto.learnblock.indices(),
                            dto.learnblock.columns(),
                            dto.learnblock.get_column("Z"),
                            dto.learnblock.min_timestamp(),
                            dto.learnblock.max_timestamp(),
                            (dto.model.subject,),
                            (zeta,))
            return reliability, PragmaticMachineLearningModel(
                meta, trained_model, dto.learnblock)

    return None, None
